@@ -24,7 +24,7 @@ except ImportError as e:
     from filter import cigar
     from filter import qualities_cigar
 
-__version__ = '0.3.14'
+__version__ = '0.4.4'
 
 def downstream_phase_detection(variants, segment, record, quality):
     """
@@ -108,13 +108,13 @@ def downstream_phase_detection(variants, segment, record, quality):
                 break
 
             if segment[idx] == parent1:
-                detection_results.append(('1', variant.start, parent1))
+                detection_results.append(('1', variant.start, parent1, query_qualities[idx]))
 
             elif segment[idx] == parent2:
-                detection_results.append(('2', variant.start, parent2))
+                detection_results.append(('2', variant.start, parent2, query_qualities[idx]))
 
             else:
-                detection_results.append(('N', variant.start, None))
+                detection_results.append(('N', variant.start, None, query_qualities[idx]))
 
     return detection_results
 
@@ -176,6 +176,7 @@ class Pair():
         self.masked_condensed = None
         self.call = None
         self.masked_call = None
+        self.phase_change_variants = None
         self.gene_conversion_len = None
 
         # quality metrics
@@ -184,12 +185,18 @@ class Pair():
         self.variants_per_haplotype = -1
         self.min_variants_in_haplotype = -1
         self.outer_bound = -1
+        self.min_end_proximity = -1
         self.variant_skew = -1
         self.mismatch_variant_ratio = -1
+        self.indels = None
+        self.indel_proximity = -1
+        self.proximate_indel_length = -1
+        self.min_base_qual = -1
+        self.mean_base_qual = -1
         self.variant_counts = None
         self.converted_haplotype = None
         self.converted_variants = None
-        self.indel_proximity = None
+        self.conversion_tract = None
 
     def __str__(self):
         """
@@ -277,7 +284,7 @@ class Pair():
             self.rec_1 = pysam.AlignedSegment.fromstring(self.rec_1, header)
             self.rec_2 = pysam.AlignedSegment.fromstring(self.rec_2, header)
 
-    def classify(self, masking=0, quality=30, vcf=None):
+    def classify(self, masking=0, quality=0, vcf=None):
         """
         Determine the type of recombination event that occursed in this read pair.
 
@@ -365,35 +372,6 @@ class Pair():
         else:
             self.call = 'no_phase_change'
 
-        if self.rec_2.reference_start + len(self.segment_2) \
-            - self.rec_1.reference_start < masking * 2:
-
-            print('Masking size too large for pair: ' + self.rec_1.query_name)
-            self.masked_condensed = None
-            self.masked_call = None
-            return
-
-        mask_start = self.rec_1.reference_start + masking
-        mask_end = self.rec_2.reference_start + len(self.segment_2) - masking
-        self.masked_detection = [
-            v for v in self.detection if mask_start < v[1] < mask_end]
-
-        # create masked_condensed from masked detection
-        # [(haplotype, beginning, end), ...]
-        self.masked_condensed = self._create_condensed(self.masked_detection)
-
-        # create list of just haplotype information no range from condensed
-        masked_haplotypes = [tupl[0] for tupl in self.masked_condensed if tupl[0] != 'N']
-
-        # classify masked_condensed
-        if len(masked_haplotypes) == 2:
-            self.masked_call = 'cross_over'
-        elif len(masked_haplotypes) == 3:
-            self.masked_call = 'gene_conversion'
-        elif len(masked_haplotypes) > 3:
-            self.masked_call = 'complex'
-        else:
-            self.masked_call = 'no_phase_change'
 
         # convert haplotypes 1/2/N to VCF sample names
         samples = vcf.samples
@@ -404,12 +382,6 @@ class Pair():
             elif tupl[0] == '2':
                 tupl[0] = samples[1]
 
-        for tupl in self.masked_condensed:
-            if tupl[0] == '1':
-                tupl[0] = samples[0]
-            elif tupl[0] == '2':
-                tupl[0] = samples[1]
-
         # set descriptive + quality attributes post-classification
         self._describe(haplotypes, vcf)
 
@@ -420,9 +392,47 @@ class Pair():
             self.gene_conversion_len = 'N/A'
             self.converted_haplotype = 'N/A'
             self.converted_variants = 'N/A'
+            self.conversion_tract = 'N/A'
 
         self.midpoint, self.relative_midpoint = self.get_midpoint()
 
+        # masking stuff
+        if self.rec_2.reference_start + len(self.segment_2) \
+            - self.rec_1.reference_start < masking * 2:
+            # masking size is too large for pair
+            self.masked_condensed = None
+            self.masked_call = None
+            return # don't set masking attributes
+
+        mask_start = self.rec_1.reference_start + masking
+        mask_end = self.rec_2.reference_start + len(self.segment_2) - masking
+        self.masked_detection = [
+            v for v in self.detection if mask_start < v[1] < mask_end]
+
+        # create masked_condensed from masked detection
+        # [(haplotype, beginning, end), ...]
+
+        self.masked_condensed = self._create_condensed(self.masked_detection)
+        for tupl in self.masked_condensed:
+            if tupl[0] == '1':
+                tupl[0] = samples[0]
+            elif tupl[0] == '2':
+                tupl[0] = samples[1]
+
+        # create list of just haplotype information in range from condensed
+        masked_haplotypes = [tupl[0] for tupl in self.masked_condensed if tupl[0] != 'N']
+
+        # classify masked_condensed
+        if len(masked_haplotypes) == 2:
+            self.masked_call = 'cross_over'
+        elif len(masked_haplotypes) == 3:
+            self.masked_call = 'gene_conversion'
+        elif len(masked_haplotypes) > 3:
+            self.masked_call = 'complex'
+        else:
+            self.masked_call = 'no_phase_change'
+
+
     def _create_condensed(self, detection):
         """
         Internal method to generate haplotype representation in
@@ -436,7 +446,9 @@ class Pair():
         condensed = []
 
         for variant in detection:
-            haplotype, location, base = variant
+            haplotype, location, base, qual = variant
+            if haplotype == 'N':
+                continue
 
             # first variant
             if len(condensed) == 0:
@@ -486,6 +498,16 @@ class Pair():
           well-supported haplotype
         - outer_bound - relative location of 'most outer' variant involved
           in a phase change
+        - min_end_proximity - lowest distance in bp between a variant involved in a
+          phase change and the end of the read it's on
+        - phase_change_variants - list of phase change variant positions
+        - indels - list of indels
+        - indel_proximity - lowest distance in bp between a variant involved in
+          a phase change and an indel
+        - proximate_indel_length - if indel_proximity != -1, size in bp of
+          nearest indel
+        - min_base_qual - minimum base qual for a phase change variant
+        - mean_base_qual - mean base qual across phase change variants
 
         Parameters
         ----------
@@ -507,35 +529,85 @@ class Pair():
         if len(self.variant_counts.values()) == 2: # only calc if both haps present
             self.variant_skew = max(self.variant_counts.values()) / min(self.variant_counts.values())
 
+        # subsequent metrics can't be calculated for non-phase-change pairs
+        if self.call == 'no_phase_change':
+            return
+
         # get the lowest number of variants a haplotype has -
         # splits variant list (e.g. ['1', '1', '2', '1']) and gets min variant count across haps
-        if 'no_phase_change' not in set([self.call, self.masked_call]):
-            self.min_variants_in_haplotype = min(
-                len(list(grouper)) for value, grouper
-                in itertools.groupby([hap for hap, position, allele in self.detection])
-                )
+        self.min_variants_in_haplotype = min(
+            len(list(grouper)) for value, grouper
+            in itertools.groupby([hap for hap, position, allele, qual in self.detection])
+            )
+
+        # get variants involved in phase change(s)
+        # used for outer_bound, min_end_proximity, indel_proximity, proximate_indel_length
+        phase_change_variants = [
+            [self.detection[i][1], self.detection[i+1][1]]
+            for i in range(len(self.detection) - 1)
+            if self.detection[i][0] != self.detection[i+1][0]]
+        self.phase_change_variants = sorted(list(set(
+            [pos for var_pair in phase_change_variants for pos in var_pair]
+        )))
+
+        # only check leftmost and rightmost variants to assign outer bound
+        if (
+            self.phase_change_variants[0] - self.condensed[0][1] < \
+            abs(self.phase_change_variants[-1] - self.condensed[-1][2])
+        ):
+            self.outer_bound = round(
+                (self.phase_change_variants[0] - self.rec_1.reference_start) / \
+                (self.condensed[-1][2] - self.condensed[0][1]), 3)
+        else:
+            self.outer_bound = round(
+                (self.phase_change_variants[-1] - self.rec_1.reference_start) / \
+                (self.condensed[-1][2] - self.condensed[0][1]), 3)
+
+        # get min_end_proximity
+        read_bounds = [
+            self.rec_1.reference_start, self.rec_1.reference_start + len(self.segment_1),
+            self.rec_2.reference_start, self.rec_2.reference_start + len(self.segment_2)]
+        self.min_end_proximity = min(
+            [abs(pos - bound) for bound in read_bounds for pos in self.phase_change_variants])
+
+        # get indels for calculation of indel attributes
+        indels = []
+        for rec in [self.rec_1, self.rec_2]:
+            idx = rec.reference_start
+            for op, basecount in rec.cigartuples:
+                if op == 0:
+                    idx += basecount
+                elif op == 1:
+                    # type, ref_start_idx, ref_end_idx, indel_length
+                    indels.append(['ins', idx, idx, basecount])
+                elif op == 2:
+                    indels.append(['del', idx, idx + basecount, basecount])
+        # overlapping reads may have the same indel register twice
+        self.indels = list(indel for indel, _ in itertools.groupby(indels)) 
+
+        if indels:
+            self.indel_proximity = min(abs(np.concatenate(
+                [np.array(self.phase_change_variants) - pos
+                 for indel in self.indels
+                 for pos in indel[1:3]])))
+
+            # length of most proximate indel
+            for indel in self.indels:
+                diffs = np.concatenate(
+                    [abs(np.array(self.phase_change_variants) - pos)
+                    for pos in indel[1:3]])
+                if self.indel_proximity in diffs:
+                    self.proximate_indel_length = indel[-1]
+                    break
+        else:
+            self.indel_proximity = -1
+            self.proximate_indel_length = -1
+
+        # base qual metrics
+        self.min_base_qual = min(site[-1] for site in self.detection)
+        self.mean_base_qual = round(
+            sum(site[-1] for site in self.detection) / len(self.detection), 2)
 
-        if self.call != 'no_phase_change':
-            phase_change_variants = [
-                [self.detection[i][1], self.detection[i+1][1]]
-                for i in range(len(self.detection) - 1)
-                if self.detection[i][0] != self.detection[i+1][0]]
-            phase_change_variants = sorted(list(set(
-                [pos for var_pair in phase_change_variants for pos in var_pair]
-            )))
-            # only check leftmost and rightmost variants
-            if (
-                phase_change_variants[0] - self.condensed[0][1] < \
-                abs(phase_change_variants[-1] - self.condensed[-1][2])
-            ):
-                self.outer_bound = round(
-                    (phase_change_variants[0] - self.rec_1.reference_start) / \
-                    (self.condensed[-1][2] - self.condensed[0][1]), 3)
-            else:
-                self.outer_bound = round(
-                    (phase_change_variants[-1] - self.rec_1.reference_start) / \
-                    (self.condensed[-1][2] - self.condensed[0][1]), 3)
-            
 
     def _describe_gene_conversion(self, vcf):
         """
@@ -548,12 +620,11 @@ class Pair():
           as determined by midpoint method
         - converted_haplotype - which haplotype was 'converted to'
         - converted_variants - which variants were converted in the tract
-        - indel_proximity - how close the nearest indel is to the converted tract, 
-          if any
+        - conversion_tract - 2-length list with bounds of conversion tract
         """
         self.gene_conversion_len = self.condensed[-1][1] - self.condensed[0][2]
 
-        # converted haplotype and variants
+        # converted haplotype, variants, and tract
         self.converted_haplotype = [
             hap for hap, count in self.variant_counts.items() 
             if count == min(self.variant_counts.values())][0]
@@ -566,30 +637,10 @@ class Pair():
         elif self.converted_haplotype == '2':
             self.converted_haplotype = vcf.samples[1]
 
-        # indel proximity
-        indels = []
-        for rec in [self.rec_1, self.rec_2]:
-            idx = rec.reference_start
-            for op, basecount in rec.cigartuples:
-                if op == 0:
-                    idx += basecount
-                elif op == 1:
-                    indels.append(['ins', idx, idx])
-                elif op == 2:
-                    indels.append(['del', idx, idx + basecount])
-        # overlapping reads may have the same indel register twice
-        indels = list(indel for indel, _ in itertools.groupby(indels)) 
-
-        conversion_tract = np.array([
+        self.conversion_tract = np.array([
             tract for tract in self.condensed 
             if tract[0] == self.converted_haplotype][0][1:])
-        if indels:
-            self.indel_proximity = min(abs(np.concatenate(
-                [np.array(indel[1:]) - np.array(conversion_tract)
-                 for indel in indels])))
-        else:
-            self.indel_proximity = 'N/A'
-        
+
 
     def _get_mismatch_variant_ratio(self):
         """
@@ -706,14 +757,14 @@ class Pair():
             self.overlap = True
 
         for site in overlapping_sites:
-            read_1_call = [(hap, pos, base) for hap, pos, base in self.detection_1 if pos == site][0]
-            read_2_call = [(hap, pos, base) for hap, pos, base in self.detection_2 if pos == site][0]
+            read_1_call = [(hap, pos, base, qual) for hap, pos, base, qual in self.detection_1 if pos == site][0]
+            read_2_call = [(hap, pos, base, qual) for hap, pos, base, qual in self.detection_2 if pos == site][0]
 
-            # if calls at same site disagree, remove variant from consideration
-            if read_1_call != read_2_call:
+            # if parental hap calls at same site disagree, remove variant from consideration
+            if read_1_call[0] != read_2_call[0]:
                 self.detection_1.remove(read_1_call)
                 self.detection_2.remove(read_2_call)
-            elif read_1_call == read_2_call:
+            elif read_1_call[0] == read_2_call[0]:
                 self.detection_2.remove(read_2_call) # only remove from right read
 
         # check if there is any overlap at all
@@ -726,10 +777,11 @@ class Pair():
         # if there's overlap, check whether overlapping sites anywhere in read disagree
         if self.overlap:
             # identify overlapping region
-            region = [self.rec_2.reference_start, self.rec_1.reference_start + len(self.segment_1)]
+            overlap_size = self.rec_1.reference_start + len(self.segment_1) - self.rec_2.reference_start
             idx = self.rec_2.reference_start - self.rec_1.reference_start
-            segment_1 = self.segment_1[idx:region[1]]
-            segment_2 = self.segment_2[0:len(segment_1)]
+            region = [idx, idx + overlap_size] # idx for segment 1
+            segment_1 = self.segment_1[region[0]:region[1]]
+            segment_2 = self.segment_2[0:overlap_size]
             if segment_1 != segment_2: # compare full strings
                 self.overlap_disagree = False
             else:
@@ -760,7 +812,7 @@ class Pair():
 
         # return midpoint if it's already been called
         if self.midpoint:
-            return self.midpoint
+            return self.midpoint, self.relative_midpoint
 
         # classify if read has not been already
         if not hasattr(self, 'call'):
@@ -769,13 +821,13 @@ class Pair():
         # simplification of results
         # [(haplotype, beginning, end), ...]
 
-        if self.masked_call == 'no_phase_change':
+        if self.call == 'no_phase_change':
             # midpoint is middle of two paired reads if no phase change
             start = self.rec_1.reference_start
             end = self.rec_2.reference_start + len(self.segment_2)
             self.midpoint = int((start + end) / 2)
             self.relative_midpoint = -1
-        elif self.masked_call == 'cross_over':
+        elif self.call == 'cross_over':
             # (X, begin, end), (Y, begin, end): end of X = beginning of Y = midpoint
             self.midpoint = self.condensed[0][2]
             # get relative midpoint
@@ -783,7 +835,7 @@ class Pair():
             end = self.condensed[-1][2]
             self.relative_midpoint = round(
                 (self.midpoint - self.rec_1.reference_start) / (end - start), 3)
-        elif self.masked_call == 'gene_conversion':
+        elif self.call == 'gene_conversion':
             # (X, begin, end), (Y, begin, end), (X', begin, end)
             # midpoint of Y tract
             self.midpoint = sum([
@@ -798,7 +850,7 @@ class Pair():
             start = self.condensed[0][2]
             end = self.condensed[-1][1]
             self.midpoint = (start + end) / 2
-            self.relative_midpoint = -1
+            self.relative_midpoint = 0.5
 
         # return in samtools format
         self.midpoint = f'{self.rec_1.reference_name}:{round(self.midpoint)}'
