@@ -46,6 +46,10 @@ def arg_parser():
         help='VCF with calls for cross of interest')
     parser.add_argument('-m', '--method', required=True, type=str, 
         help='Filtering method to use (midpoint/overlap/nuclear) - see docs')
+    parser.add_argument('-l', '--read_length_filter', required=False, type=int, 
+        help='Filter out false positive reads below specified length threshold (default 0)')
+    parser.add_argument('-q', '--base_qual_filter', required=False, type=int, 
+        help='Ignore phase change variants in fp reads below specified base qual filter (default 0)')
     parser.add_argument('--false_bed_out', required=False, type=str, 
         help='Where to save false phase change bed file [optional]')
     parser.add_argument('--false_bed_in', required=False, type=str, 
@@ -54,12 +58,13 @@ def arg_parser():
         help='Path to log file')
     parser.add_argument('-o', '--out', required=True, type=str,
         help='File to write filtered reads to')
-    parser.add_argument('--version', action='version', version='readcomb 0.3.14')
+    parser.add_argument('--version', action='version', version='readcomb 0.4.4')
 
     return parser
 
 class BedGenerator():
-    def __init__(self, false_plus, false_minus, vcf_fname, method, bed_fname):
+    def __init__(self, false_plus, false_minus, vcf_fname, method,
+        read_length_filter, base_qual_filter, bed_fname):
         """Generate BED file of false positive midpoints
 
         BED format will depend on what filtering method is requested -
@@ -81,6 +86,10 @@ class BedGenerator():
             path to VCF used to call recombination events
         method : str
             one of 'midpoint', 'overlap', or 'nuclear'
+        read_length_filter : int
+            filter out false positive reads below this size (default 0)
+        base_qual_filter : int
+            ignore informative sites in reads below this base quality (default 0)
         bed_fname : str
             path to bed outfile
         """
@@ -90,6 +99,16 @@ class BedGenerator():
         self.vcf_fname = vcf_fname
 
         self.method = method
+        if read_length_filter:
+            self.read_length_filter = read_length_filter
+        else:
+            self.read_length_filter = 0
+
+        if base_qual_filter:
+            self.base_qual_filter = base_qual_filter
+        else:
+            self.base_qual_filter = 0
+
         self.bed_fname = bed_fname
         if not self.bed_fname: # if fname not provided - generate temp bed
             self.temp = True
@@ -130,16 +149,21 @@ class BedGenerator():
         # iterate through false positives
         for i, reader in enumerate([plus_reader, minus_reader]):
             for pair in tqdm(reader, desc=f'parental false positive bed generation {i+1}'):
-                pair.classify(masking=0)
+                pair.classify(masking=0, quality=self.base_qual_filter)
                 if pair.call == 'no_phase_change': # ignore if no phase change
                     continue
+                if not (
+                    len(pair.rec_1.query_sequence) > self.read_length_filter and
+                    len(pair.rec_2.query_sequence) > self.read_length_filter
+                ):
+                    continue
                 else:
                     detection_filt = [
                         variant for variant in pair.detection 
                         if variant[0] != 'N']
                     prev_hap, prev_pos = detection_filt[0][:2] # set to first hap
                     for variant in detection_filt:
-                        hap, pos, base = variant
+                        hap, pos, base, qual = variant
                         if hap == prev_hap:
                             prev_pos = pos # update variable
                             continue
@@ -178,9 +202,14 @@ class BedGenerator():
         # iterate through false positives
         for i, reader in enumerate([plus_reader, minus_reader]):
             for pair in tqdm(reader, desc=f'parental false positive bed generation {i+1}/2'):
-                pair.classify(masking=0)
+                pair.classify(masking=0, quality=self.base_qual_filter)
                 if pair.call == 'no_phase_change': # ignore if no phase change
                     continue
+                if not (
+                    len(pair.rec_1.query_sequence) > self.read_length_filter and
+                    len(pair.rec_2.query_sequence) > self.read_length_filter
+                ):
+                    continue
                 else:
                     bed_writer.writerow({
                         '#chrom': pair.rec_1.reference_name,
@@ -219,9 +248,14 @@ class BedGenerator():
         # iterate through false positives
         for i, reader in enumerate([plus_reader, minus_reader]):
             for pair in tqdm(reader, desc=f'parental false positive bed generation {i+1}/2'):
-                pair.classify(masking=0)
+                pair.classify(masking=0, quality=self.base_qual_filter)
                 if pair.call == 'no_phase_change': # ignore if no phase change
                     continue
+                if not (
+                    len(pair.rec_1.query_sequence) > self.read_length_filter and
+                    len(pair.rec_2.query_sequence) > self.read_length_filter
+                ):
+                    continue
                 else:
                     bed_writer.writerow({
                         '#chrom': pair.rec_1.reference_name,
@@ -319,7 +353,7 @@ class BedGenerator():
         if tabix_bed:
             print('[readcomb] bgzip and tabix on bed file...')
             bgzip_proc = subprocess.run(['bgzip', self.bed_fname])
-            tabix_proc = subprocess.run(['tabix', self.bed_fname + '.gz'])
+            tabix_proc = subprocess.run(['tabix', '-b2', '-e3', self.bed_fname + '.gz'])
             print('[readcomb] complete.')
 
 
@@ -389,7 +423,7 @@ class FalsePositiveFilterer():
 
         for pair in tqdm(reader, desc=f'filtering {self.fname}'):
             log_row['total_pairs'] += 1
-            pair.classify(masking=0)
+            pair.classify(masking=0, quality=0)
             if pair.call == 'no_phase_change':
                 log_row['no_phase_change'] += 1
                 continue
@@ -439,7 +473,7 @@ class FalsePositiveFilterer():
 
     def _filter_midpoint(self, pair, chrom, start, end, false_lookup):
         # reduce pair.detection output for comparison
-        detection_reduced = [(hap, pos) for hap, pos, base in pair.detection]
+        detection_reduced = [(hap, pos) for hap, pos, base, qual in pair.detection]
         # iterate over false reads
         for false_read in false_lookup:
             chrom, pos1, pos2, hap1, hap2, parent = false_read.split('\t')
@@ -495,6 +529,8 @@ def handle_bed(args):
         false_minus=args.false_minus,
         vcf_fname=args.vcf,
         method=args.method,
+        read_length_filter=args.read_length_filter,
+        base_qual_filter=args.base_qual_filter,
         bed_fname=args.false_bed_out)
 
     bed_generator.generate_bed() # will use method specified above
